#!/bin/bash
# Authors:
#    LT Thomas <ltjr@ti.com>
#    Chase Maupin
# create-sdcard.sh v0.3
# 
# Changed: Ratti Ezio
# fast-create-sdcard.sh v0.2

# This distribution contains contributions or derivatives under copyright
# as follows:
#
# Copyright (c) 2010, Texas Instruments Incorporated
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# - Neither the name of Texas Instruments nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

username="architech"
groupname="architech"
export START_DIR=`pwd`
export YOCTO_IMAGES=/home/architech/architech_sdk/architech/pengwyn/yocto/build/tmp/deploy/images


#######################################
# check prerequisite
if [[ ! -s "$START_DIR/create-nand-fs.sh" ]]; then
		cat << EOM
+------------------------------------------------------------------------------+
|                                                                              |
| Error: script 'create-nand-fs.sh' doesn't exist.                             |
|                                                                              |
+------------------------------------------------------------------------------+

EOM
		read -p 'Press ENTER to finish' FILEPATHOPTION
		exit
fi

#
if [[ ! -s "$START_DIR/bar" ]]; then
		cat << EOM
+------------------------------------------------------------------------------+
|                                                                              |
| Error: script 'bar' doesn't exist.                                           |
|                                                                              |
+------------------------------------------------------------------------------+

EOM
		read -p 'Press ENTER to finish' FILEPATHOPTION
		exit
fi

#
if [[ "a$1" != "a" && "$1" != "custom" ]]; then
	echo Set source folder to $1
	YOCTO_IMAGES=$1
fi
if [ ! -s "$YOCTO_IMAGES" ]; then
	cat << EOM
+------------------------------------------------------------------------------+
|                                                                              |
| Source folder doesn't exists: check path.                                    |
| $YOCTO_IMAGES
|                                                                              |
+------------------------------------------------------------------------------+

EOM
	read -p 'Press ENTER to finish' FILEPATHOPTION
	exit
fi

# Determine the absolute path to the executable
# EXE will have the PWD removed so we can concatenate with the PWD safely
PWD=`pwd`
EXE=`echo $0 | sed s=$PWD==`
EXEPATH="$PWD"/"$EXE"
clear
cat << EOM

+------------------------------------------------------------------------------+
|                                                                              |
| This script will create a bootable SD card.                                  |
| The script must be run with root permissions.                                |
|                                                                              |
+------------------------------------------------------------------------------+

EOM


#######################################
AMIROOT=`whoami | awk {'print $1'}`
if [ "$AMIROOT" != "root" ] ; then

	echo "	**** Error *** must run script with sudo"
	echo ""
	read -p 'Press ENTER to finish' FILEPATHOPTION
	exit
fi

THEPWD=$EXEPATH
PARSEPATH=`echo $THEPWD | grep -o '.*ti-sdk.*.[0-9]/'`

if [ "$PARSEPATH" != "" ] ; then
	PATHVALID=1
else
	PATHVALID=0
fi


#######################################
#Percentage function
untar_progress ()
{
    TARBALL=$1;
    DIRECTPATH=$2;
    BLOCKING_FACTOR=$(($(gzip --list ${TARBALL} | sed -n -e "s/.*[[:space:]]\+[0-9]\+[[:space:]]\+\([0-9]\+\)[[:space:]].*$/\1/p") / 51200 + 1));
    tar --blocking-factor=${BLOCKING_FACTOR} --checkpoint=1 --checkpoint-action='ttyout=Written %u%  \r' -zxf ${TARBALL} -C ${DIRECTPATH}
}


#######################################
#copy/paste programs
cp_progress ()
{
	CURRENTSIZE=0
	while [ $CURRENTSIZE -lt $TOTALSIZE ]
	do
		TOTALSIZE=$1;
		TOHERE=$2;
		CURRENTSIZE=`sudo du -c $TOHERE | grep total | awk {'print $1'}`
		echo -e -n "$CURRENTSIZE /  $TOTALSIZE copied \r"
		sleep 1
	done
}


#######################################
# find the available SD cards

mkdir -p $START_DIR/temp
ROOTDRIVE=`mount | grep 'on / ' | awk {'print $1'} |  cut -c6-8`
cat /proc/partitions | grep -v $ROOTDRIVE | grep '\<sd.\>' | grep -n '' > $START_DIR/temp/sd_list.txt
SIZE=$(stat --printf="%s" $START_DIR/temp/sd_list.txt)

if [ "$SIZE" -eq "0" ]; then
	cat << EOM
+------------------------------------------------------------------------------+
| ERROR! No drive available                                                    |
+------------------------------------------------------------------------------+

EOM
	read -p 'Press ENTER to finish' FILEPATHOPTION
	exit
fi


#######################################
# show available SD card and select which to use

cat << EOM

+------------------------------------------------------------------------------+
| LIST OF AVAILABLE DRIVES:                                                    |
+------------------------------------------------------------------------------+

EOM

ROOTDRIVE=`mount | grep 'on / ' | awk {'print $1'} |  cut -c6-8`
echo "#  major   minor    size   name "
cat /proc/partitions | grep -v $ROOTDRIVE | grep '\<sd.\>' | grep -n ''
echo " "

ENTERCORRECTLY=0
while [ $ENTERCORRECTLY -ne 1 ]
do
	read -p 'Enter Device Number #: ' DEVICEDRIVENUMBER
	echo " "
	DEVICEDRIVENAME=`cat /proc/partitions | grep -v 'sda' | grep '\<sd.\>' | grep -n '' | grep "${DEVICEDRIVENUMBER}:" | awk '{print $5}'`

	DRIVE=/dev/$DEVICEDRIVENAME
	DEVICESIZE=`cat /proc/partitions | grep -v 'sda' | grep '\<sd.\>' | grep -n '' | grep "${DEVICEDRIVENUMBER}:" | awk '{print $4}'`


	if [ -n "$DEVICEDRIVENAME" ]
	then
		ENTERCORRECTLY=1
	else
		echo "Invalid selection"
	fi
done

echo "$DEVICEDRIVENAME was selected"
#Check the size of disk to make sure its under 16GB
if [ $DEVICESIZE -gt 17000000 ] ; then
cat << EOM

################################################################################
		**********WARNING**********
	Selected Device is greater then 16GB
	Continuing past this point will erase data from device
	Double check that this is the correct SD Card
################################################################################

EOM
	ENTERCORRECTLY=0
	while [ $ENTERCORRECTLY -ne 1 ]
	do
		read -p 'Would you like to continue [y/n] : ' SIZECHECK
		echo ""
		echo " "
		ENTERCORRECTLY=1
		case $SIZECHECK in
		"y")  ;;
		"n")  exit;;
		*)  echo "Please enter y or n";ENTERCORRECTLY=0;;
		esac
		echo ""
	done
fi

ALREADY_FORMATTED=1
unmounted1=`df | grep '\<'$DEVICEDRIVENAME'1\>' | awk '{print $6}'`
unmounted2=`df | grep '\<'$DEVICEDRIVENAME'2\>' | awk '{print $6}'`

if [ "$unmounted1" != "/media/boot" ]; then
	ALREADY_FORMATTED=0
fi
if [ "$unmounted2" != "/media/rootfs" ]; then
	ALREADY_FORMATTED=0
fi

if [ "$ALREADY_FORMATTED" = "0" ]; then
	if [ -d "/dev/disk/by-label" ]; then
		unmounted1=`ls -l /dev/disk/by-label/* | grep ''$DEVICEDRIVENAME'1' | awk '{print $8}'`
		unmounted2=`ls -l /dev/disk/by-label/* | grep ''$DEVICEDRIVENAME'2' | awk '{print $8}'`

	 	if [[ "$unmounted1" = "/dev/disk/by-label/boot" && "$unmounted2" = "/dev/disk/by-label/rootfs" ]]; then
	    		ALREADY_FORMATTED=1
		fi
	fi
fi


DRIVE=/dev/$DEVICEDRIVENAME

#echo "Checking the device is unmounted"
#unmount drives if they are mounted
unmounted1=`df | grep '\<'$DEVICEDRIVENAME'1\>' | awk '{print $1}'`
unmounted2=`df | grep '\<'$DEVICEDRIVENAME'2\>' | awk '{print $1}'`
unmounted3=`df | grep '\<'$DEVICEDRIVENAME'3\>' | awk '{print $1}'`
if [ -n "$unmounted1" ]; then
	#echo " unmounted ${DRIVE}1"
	sudo umount -f ${DRIVE}1
fi
if [ -n "$unmounted2" ]; then
	#echo " unmounted ${DRIVE}2"
	sudo umount -f ${DRIVE}2
fi
if [ -n "$unmounted3" ]; then
	#echo " unmounted ${DRIVE}3"
	sudo umount -f ${DRIVE}3
fi
echo ""
# check to see if the device is already partitioned
SIZE1=`cat /proc/partitions | grep -v 'sda' | grep '\<'$DEVICEDRIVENAME'1\>'  | awk '{print $3}'`
SIZE2=`cat /proc/partitions | grep -v 'sda' | grep '\<'$DEVICEDRIVENAME'2\>'  | awk '{print $3}'`
echo "${DEVICEDRIVENAME}1  ${DEVICEDRIVENAME}2"
echo $SIZE1 $SIZE2
echo ""

PARTITION=0
if [ -n "$SIZE1" -a -n "$SIZE2" ] ; then
	if  [ "$SIZE1" -gt "72000" -a "$SIZE2" -gt "700000" ]; then
		PARTITION=1

		if [ -z "$SIZE3" -a -z "$SIZE4" ]; then
			#Detected 2 partitions
			PARTS=2
		elif [ "$SIZE3" -gt "1000" -a -z "$SIZE4" ]; then
			#Detected 3 partitions
			PARTS=3
		else
			echo "SD Card is not correctly partitioned"
			PARTITION=0
		fi
	fi
else
	echo "SD Card is not correctly partitioned"
	PARTITION=0
	PARTS=0
fi


if [[ "$PARTITION" -eq "1" && "$ALREADY_FORMATTED" -eq "1" ]]; then
	#echo "FAST PARTITIONS"
	PARTS=2
	FAST_PARTITIONS=1
else
	FAST_PARTITIONS=0

#######################################
#Partition is found
	if [ "$PARTITION" -eq "1" ]; then
cat << EOM

################################################################################

   Detected device has $PARTS partitions already

################################################################################

EOM

		ENTERCORRECTLY=0
		while [ $ENTERCORRECTLY -ne 1 ]
		do
			read -p 'Would you like to re-partition the drive anyways [y/n] : ' CASEPARTITION
			echo ""
			echo " "
			ENTERCORRECTLY=1
			case $CASEPARTITION in
			"y")  echo "Now partitioning $DEVICEDRIVENAME ...";PARTITION=0;;
			"n")  echo "Skipping partitioning";;
			*)  echo "Please enter y or n";ENTERCORRECTLY=0;;
			esac
			echo ""
		done
	fi

#######################################
#Partition is not found, create 2 partitions
	if [ "$PARTITION" -eq "0" ]; then

		PARTITION=2

#Section for partitioning the drive
# Set the PARTS value as well
		PARTS=2
cat << EOM

+------------------------------------------------------------------------------+
|		Now making 2 partitions					       |
+------------------------------------------------------------------------------+
EOM
		dd if=/dev/zero of=$DRIVE bs=1024 count=1
		sync
		sync
		sync

		SIZE=`fdisk -l $DRIVE | grep Disk | awk '{print $5}'`

		echo DISK SIZE - $SIZE bytes

		CYLINDERS=`echo $SIZE/255/63/512 | bc`

		sfdisk -D -H 255 -S 63 -C $CYLINDERS $DRIVE << EOF
,9,0x0C,*
10,,,-
EOF

		cat << EOM

+------------------------------------------------------------------------------+
|		Partitioning Boot					       |
+------------------------------------------------------------------------------+
EOM
		if [ ! -n "${DRIVE}1" ]; then
			echo "Error: wrong ${DRIVE}1 partition, repeat creation"
			read -p 'Press ENTER to finish' FILEPATHOPTION
			exit
		fi
		mkfs.vfat -F 32 -n "boot" ${DRIVE}1
cat << EOM

+------------------------------------------------------------------------------+
|		Partitioning rootfs					       |
+------------------------------------------------------------------------------+
EOM
		if [ ! -n "${DRIVE}2" ]; then
			echo "Error: wrong ${DRIVE}2 partition, repeat creation"
			read -p 'Press ENTER to finish' FILEPATHOPTION
			exit
		fi
		mkfs.ext3 -L "rootfs" ${DRIVE}2
		sync
		sync
		INSTALLSTARTHERE=n
	fi
fi


######################################################################
# Add directories for images

mkdir -p $START_DIR/tmp
export PATH_TO_SDBOOT=$START_DIR/boot
export PATH_TO_SDROOTFS=$START_DIR/rootfs
export PATH_TO_TMP_DIR=$START_DIR/tmp


echo "Mount the partitions "
mkdir -p $PATH_TO_SDBOOT
mkdir -p $PATH_TO_SDROOTFS

sudo mount -t vfat ${DRIVE}1 $PATH_TO_SDBOOT/
sudo mount -t ext3 ${DRIVE}2 $PATH_TO_SDROOTFS/

echo "Emptying partitions "
sudo rm -rf  $PATH_TO_SDBOOT/*
sudo rm -rf  $PATH_TO_SDROOTFS/*

echo "Syncing...."
echo ""
sync
sync
sync


############################################################################################
############################################################################################
############################################## SELECT FILES TO DOWNLOAD

#if [ "$FAST_PARTITIONS" = "0" ]; then
if [ "$1" = "custom" ]; then

	cat << EOM
################################################################################

  For Boot partition

  If files are located in Tarball write complete path including the file name.
      e.x. $:  /home/architech/MyCustomTars/boot.tar.gz

  and the beginning of the files should be labeled with MLO, u-boot, uImage
      i.e.   test_MLO_image must be labeled as MLO_test_image

################################################################################

EOM
	ENTERCORRECTLY=0
	while [ $ENTERCORRECTLY -ne 1 ]
	do
		read -e -p 'Enter path for Boot Partition : '  BOOTUSERFILEPATH

		echo ""
		ENTERCORRECTLY=1
		if [ -f $BOOTUSERFILEPATH ]
		then
			echo "File exists"
			echo ""
		elif [ -d $BOOTUSERFILEPATH ]
		then
			echo "Directory exists"
			echo ""
			echo "This directory contains:"
			ls $BOOTUSERFILEPATH
			echo ""
			read -p 'Is this correct? [y/n] : ' ISRIGHTPATH
				case $ISRIGHTPATH in
				"y") ;;
				"n") ENTERCORRECTLY=0;;
				*)  echo "Please enter y or n";ENTERCORRECTLY=0;;
				esac
		else
			echo "Invalid path make sure to include complete path"

			ENTERCORRECTLY=0
		fi
	done

cat << EOM


################################################################################

   For Rootfs partition

   If files are located in Tarball write complete path including the file name.
      e.x. $:  /home/architech/MyCustomTars/rootfs.tar.gz

################################################################################

EOM
	ENTERCORRECTLY=0
	while [ $ENTERCORRECTLY -ne 1 ]
	do
		read -e -p 'Enter path for Rootfs Partition : ' ROOTFSUSERFILEPATH
		echo ""
		ENTERCORRECTLY=1
		if [ -f $ROOTFSUSERFILEPATH ]
		then
			echo "File exists"
			echo ""
		elif [ -d $ROOTFSUSERFILEPATH ]
		then
			echo "This directory contains:"
			ls $ROOTFSUSERFILEPATH
			echo ""
			read -p 'Is this correct? [y/n] : ' ISRIGHTPATH
				case $ISRIGHTPATH in
				"y") ;;
				"n") ENTERCORRECTLY=0;;
				*)  echo "Please enter y or n";ENTERCORRECTLY=0;;
				esac

		else
			echo "Invalid path make sure to include complete path"

			ENTERCORRECTLY=0
		fi
	done
	echo ""


	# Check if user entered a tar or not for Boot
	ISBOOTTAR=`ls $BOOTUSERFILEPATH | grep .tar.gz | awk {'print $1'}`
	if [ -n "$ISBOOTTAR" ]
	then
		BOOTPATHOPTION=2
	else
		BOOTPATHOPTION=1
		BOOTFILEPATH=$BOOTUSERFILEPATH
		MLO=`ls $BOOTFILEPATH | grep MLO | awk {'print $1'}`
		UIMAGE=`ls $BOOTFILEPATH | grep uImage | awk {'print $1'}`
		BOOTIMG=`ls $BOOTFILEPATH | grep u-boot | grep .img | awk {'print $1'}`
		BOOTBIN=`ls $BOOTFILEPATH | grep u-boot | grep .bin | awk {'print $1'}`
	fi

	#Check if user entered a tar or not for Rootfs
	ISROOTFSTAR=`ls $ROOTFSUSERFILEPATH | grep .tar.gz | awk {'print $1'}`
	if [ -n "$ISROOTFSTAR" ]
	then
		ROOTFSPATHOPTION=2
	else
		ROOTFSPATHOPTION=1
		ROOTFSFILEPATH=$ROOTFSUSERFILEPATH
	fi


cat << EOM
+------------------------------------------------------------------------------+
|	Copying files now... will take minutes				       |
+------------------------------------------------------------------------------+

Copying boot partition
EOM

	echo ""
	#copy boot files out of board support
	if [ "$MLO" != "" ] ; then
		cp $BOOTFILEPATH/$MLO $PATH_TO_SDBOOT/MLO
		echo "MLO copied"
	else
		echo "MLO file not found"
	fi

	echo ""

	if [ "$BOOTBIN" != "" ] ; then
		cp $BOOTFILEPATH/$BOOTBIN $PATH_TO_SDBOOT/u-boot.bin
		echo "u-boot.bin copied"
	else
		echo "u-boot.bin file not found"
	fi

	echo ""

	if [ "$BOOTIMG" != "" ] ; then
		cp $BOOTFILEPATH/$BOOTIMG $PATH_TO_SDBOOT/u-boot.img
		echo "u-boot.img copied"
	else
		echo "u-boot.img file not found"
	fi

	echo ""

	if [ "$UIMAGE" != "" ] ; then
		cp $BOOTFILEPATH/$UIMAGE $PATH_TO_SDBOOT/uImage
		echo "uImage copied"
	else
		echo "uImage file not found"
	fi

	echo ""
	sync

	echo "Copying rootfs System partition"

	TOTALSIZE=`sudo du -c $ROOTFSUSERFILEPATH/* | grep total | awk {'print $1'}`
	sudo cp -r $ROOTFSUSERFILEPATH/* $PATH_TO_SDROOTFS & cp_progress $TOTALSIZE $PATH_TO_SDROOTFS

else

############################################################################################
############################################################################################
############################################## FAST PARTITIONING AND COPYING

	mkdir -p $START_DIR/temp
	mkdir -p $START_DIR/temp/boot
	mkdir -p $START_DIR/temp/rootfs

	if [ -f $YOCTO_IMAGES/MLO ]; then
		cp $YOCTO_IMAGES/MLO $START_DIR/temp/boot/MLO
	else
		cat << EOM
+------------------------------------------------------------------------------+
| ERROR! No 'MLO' boot loader available                                        |
+------------------------------------------------------------------------------+

EOM
		read -p 'Press ENTER to finish' FILEPATHOPTION
		exit
	fi


	if [ -f $YOCTO_IMAGES/u-boot-pengwyn.img ]; then
		cp $YOCTO_IMAGES/u-boot-pengwyn.img $START_DIR/temp/boot/u-boot.img
	else
		cat << EOM
+------------------------------------------------------------------------------+
| ERROR! No 'u-boot-pengwyn.img' available                                     |
+------------------------------------------------------------------------------+

EOM
		read -p 'Press ENTER to finish' FILEPATHOPTION
		exit
	fi


	if [ -f $YOCTO_IMAGES/uImage.bin ]; then
		cp $YOCTO_IMAGES/uImage.bin $START_DIR/temp/boot/uImage
	else
		cat << EOM
+------------------------------------------------------------------------------+
| ERROR! No 'uImage.bin' Linux kernel available                                |
+------------------------------------------------------------------------------+

EOM
		read -p 'Press ENTER to finish' FILEPATHOPTION
		exit
	fi


	ls -1cr $YOCTO_IMAGES/*.rootfs.tar.gz > $START_DIR/temp/fs_list.txt 2>/dev/null
	FS_NAME=$(tail -n 1 $START_DIR/temp/fs_list.txt)
	#echo $FS_NAME
	if [ -s "$FS_NAME" ]; then
		cp $FS_NAME $START_DIR/temp/rootfs/
	else
		cat << EOM
+------------------------------------------------------------------------------+
| ERROR! No rootfs available
| $YOCTO_IMAGES/*.rootfs.tar.gz
+------------------------------------------------------------------------------+

EOM
		read -p 'Press ENTER to finish' FILEPATHOPTION
		exit
	fi


	ls -1cr $YOCTO_IMAGES/module* > $START_DIR/temp/modules_list.txt 2>/dev/null
	MODULES_NAME=$(tail -n 1 $START_DIR/temp/modules_list.txt)
	#echo $MODULES_NAME
	if [ -s "$MODULES_NAME" ]; then
		cp $MODULES_NAME $START_DIR/temp/rootfs/
	else
		echo "No modules available"
	fi

	cat << EOM
+------------------------------------------------------------------------------+
|	Copying files now... will take minutes				       |
+------------------------------------------------------------------------------+

Copying boot partition
EOM

	cp $START_DIR/temp/boot/* $PATH_TO_SDBOOT
	sync

	echo "Copying file system partition: $FS_NAME"
	untar_progress $FS_NAME $PATH_TO_SDROOTFS
	$START_DIR/bar -n -o $PATH_TO_SDROOTFS/home/root/yocto_rootfs.tar.gz $FS_NAME
	$START_DIR/bar -n -o $PATH_TO_SDROOTFS/home/root/create-nand-fs.sh $START_DIR/create-nand-fs.sh
	chmod +x $PATH_TO_SDROOTFS/home/root/create-nand-fs.sh

	if [ -s "$MODULES_NAME" ]; then
		echo "Copying modules"
		untar_progress $MODULES_NAME $PATH_TO_SDROOTFS
		$START_DIR/bar -n -o $PATH_TO_SDROOTFS/home/root/yocto_modules.tar.gz $MODULES_NAME
	fi
fi

echo "Syncing...."
sync
sync
sync
sync
sync
sync
sync
sync

echo "Un-mount the partitions "
sudo umount -f $PATH_TO_SDBOOT
sudo umount -f $PATH_TO_SDROOTFS

echo "Remove created temp directories "
sudo rm -rf $PATH_TO_TMP_DIR
sudo rm -rf $PATH_TO_SDROOTFS
sudo rm -rf $PATH_TO_SDBOOT

echo "Operation Finished"
echo " "
read -p 'Press ENTER to finish' FILEPATHOPTION

